#ifndef __DATASTRUCTS_H
#define __DATASTRUCTS_H

enum {
    QTREE_THRESHOLD = 6
};


typedef struct Pos Pos;
typedef struct Object Object;
typedef struct ListNode ListNode;
typedef struct List List;
typedef struct QTree QTree;
typedef struct Map Map;
typedef struct BinTree BinTree;

struct Pos {
    int x, y;
};

struct Object {
/*    unsigned char type;*/   /* might be needed if we mix different types */
    Pos pos;
};

struct ListNode {
    void *p;
    List *list;
    ListNode *prev, *next;
};

struct List {
    unsigned short size;
    ListNode *first, *last;
};

struct QTree {
    Pos pos;
    unsigned int width, size;
    List *content;
    QTree *parent, *nw, *ne, *sw, *se;
};

struct Map {
    BinTree *tree;
};

struct BinTree {
    char *key;
    void *value;
    BinTree *left, *right;
};


inline Pos mkPos(int x, int y);
inline Pos posAdd(Pos a, Pos b);
inline Pos posSub(Pos a, Pos b);
int posCmp(Pos a, Pos b);

ListNode * addListNode(List *l);
void delListNode(ListNode *n);
List * addList(void);
void delList(List *l);
void swapList(List **a, List **b);

QTree * addQRoot(void);
void delQTree(QTree *t);
void insertObject(QTree **tp, Object *o);
List * getObjectsAt(QTree *t, Pos pos);
Object * getObjectAt(QTree *t, Pos pos);

Map * addMap(void);
void delMap(Map *m);
int mapInsert(Map *m, char *key, void *value);
void * mapLookup(Map *m, char *key);

#endif /* __DATASTRUCTS_H */
