/* debug control/output */
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>

#include "ui.h"
#include "lidl.h"

FILE *debugfile;

void debugInit( void )
{
    /* Used to initialise the debuging specific stuff */
#ifdef DEBUG
    debugfile = fopen( "debug.log", "w" );
#endif
}

void debugStop( void )
{
#ifdef DEBUG
    /* shutdown the debuger */
    fclose( debugfile );
#endif
}

void debugDie( const char *file, int line, char *fmt, ... )
{
    /* This code should only be used if something went really really wrong
       IG: We crashed!
       NOTE: we should ONLY stop stuff that really has to. */

    va_list ap;

    uiStop(); /* really needed */

    fprintf( stderr, "CRASH in %s:%d\nDescription:\n", file, line );
    va_start( ap, fmt );
    vfprintf( stderr, fmt, ap );
    va_end( ap );
    fprintf( stderr, "\n" );

    exit( EXIT_FAILURE );
}

void debugPrintLine( const char *filename, char *fmt, ... )
{
    /* prints the debug string as the ui wishes
       NOTE: This function should be used for debug messages */

#ifdef DEBUG
    va_list ap;
  
    fprintf( debugfile, "%s: ", filename );
    va_start( ap, fmt );
    vfprintf( debugfile, fmt, ap );
    va_end( ap );
    fprintf( debugfile, "\n" );
#endif
}

void debugInteractive( void )
{
    /* This is called from game.c when the player requests to enter
       the debug menu etc, this will talk to the ui etc and setup,
       variables etc.

       The player should be able later on to load levels, spawn monsters,
       change his stats etc. */
#ifdef DEBUG
    char debugCommand[20];
    int x, y;
    Pos p;

    uiPrint( "DBG: current commands: teleport" );
    uiInputString( debugCommand, sizeof( debugCommand ), "DBG: command" );
    
    if( strcmp( debugCommand, "teleport" ) == 0 )
    {
        uiInputString( debugCommand, sizeof( debugCommand ),
                       "DBG: where (x,y)" );
        if( sscanf( debugCommand, "%d,%d", &x, &y ) == 2 )
        {
            p = mkPos( x, y );
            playerMove( p ); /* does not work :/ */
            uiPrint( "DBG: x=%d, y=%d", p.x, p.y );
        }
        else
            uiPrint( "DBG: no no..." );
    }
    else
        uiPrint( "DBG: '%s' command not found", debugCommand );
#endif
}
