%{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "datastructs.h"
#include "lldata.h"

int yylex(void);
void yyerror(const char *s);

%}

%union {
   char *str;
   int intnum;
   double floatnum;
   LLValue *val;
}

%token INTEGER FLOAT STRING ID
%token INCLUDE DICE

%type <intnum> INTEGER
%type <floatnum> FLOAT
%type <str> STRING ID
%type <val> expr simple position

%left '(' ')'

%%

input:
    blocks
;

blocks:
    block
  | block blocks
;

block:
    INCLUDE STRING                 { /* parse it immediately */ }
  | ID '{' declarations '}' ';'
  | ID '(' tlist ')' '{' declarations '}' ';'
;

tlist:
    ID
  | ID ',' tlist
;

declarations:
    declaration
  | declaration declarations
;

declaration:
    ID '=' expr ';'
;

expr:
    simple                        { $$ = $1; }
  | '<' position '>'              { $$ = $2; }
;

simple:
    INTEGER                       { $$ = embedInt($1); }
  | FLOAT                         { $$ = embedFloat($1); }
  | STRING                        { $$ = embedString($1); }
  | ID                            { $$ = embedSymbol($1); }
;

position:
    expr                          { $$ = NULL; }
  | expr ',' position             { $$ = $1; }
;

%%

/*
NNode * make_stringnode(char *data)
{
   NValue *node;

   node = new_nvalue(data, strlen(data) + 1, NT_STRING);
 
   return &node->nodehead;
}
*/

void yyerror(const char *s)
{
    fprintf(stderr, "parse error: %s\n", s);
    exit(EXIT_FAILURE);
}
