/* the world */
#include <stdio.h>
#include <stdlib.h>

#include "lidl.h"
#include "datastructs.h"
#include "world.h"
#include "ui.h"

struct WorldObject {
    Object obj;
    int type;
};
typedef struct WorldObject WorldObject;

QTree *worldMap;

WorldObject * addWorldObject( QTree **t, Pos pos, int type )
{
    WorldObject *o = (WorldObject *)malloc( sizeof( WorldObject ) );
    o->type = type;
    o->obj.pos = pos;
    insertObject( t, &o->obj );
    return o;
}

WorldObject * worldAt( Pos pos )
{
    WorldObject *o = (WorldObject *) getObjectAt (worldMap, pos );
    if( !o )
	return addWorldObject( &worldMap, pos, worldWALL ); /* default worldobject */
    return o;
}

void worldInit( void )
{
    /* init's the world and places a bunch of evil's here and there */
    worldMap = addQRoot();

    /* clear the cave */
    Pos p;
    for( p.x = 0; p.x < 6; p.x++ )
	for( p.y = 0; p.y < 6; p.y++ )
	    addWorldObject( &worldMap, p, worldEMPTY );
}

void worldSpawnPlayer( Pos *p )
{
    /* This function should ONLY be called when a new game starts
       And ONLY from playerInit
       Will maybe be converted to a generic worldSpawn function later */

    *p = mkPos( 1, 1 );
}

int worldCheckSpot( Pos p )
{
    /* Checks if spot x, y, z is empty or what it is.
       NOTE: Monsters and Player(s) is not handeled here.
       return codes:
       -1 Error worldERROR
       0  Empty worldEMPTY
       1  Wall  worldWALL
       2  Hole  worldHOLE */

    int spotcode = worldAt( p )->type;

    debugPrint( "spotcode = %d", spotcode );

    if( spotcode < worldEMPTY || spotcode > worldHOLE )
    {
	die( "spotcode: %d pos: (%d,%d)", spotcode, p.x, p.y );
	spotcode = worldERROR;
	debugPrint( "spotcode error!" );
    }

    return spotcode;
}

Pos worldGetMove( Pos pos, int direction )
{
    /* Returns the position for the move
       x, y is the original position of the thing
       direction is n, e, w, s used to calculate what spot we
       are talking about, sadly this code will most likely need
       a rewrite when the third dimension is introduced
       (or other stuff like ne)

       FIXME: we need a system for returns so we will be able to
       warn the called that the move failed */

    Pos dPos;

    switch( direction )
    {
    case uiMOVEN:
	dPos = mkPos(  0, -1 );
	break;
    case uiMOVES:
	dPos = mkPos(  0,  1 );
	break;
    case uiMOVEW:
	dPos = mkPos( -1,  0 );
	break;
    case uiMOVEE:
	dPos = mkPos(  1,  0 );
	break;
    case uiMOVENW:
	dPos = mkPos( -1, -1 );
	break;
    case uiMOVENE:
	dPos = mkPos( 1,  -1 );
	break;
    case uiMOVESW:
	dPos = mkPos( -1,  1 );
	break;
    case uiMOVESE:
	dPos = mkPos(  1,  1 );
	break;
    }

    return posAdd( pos, dPos );
}
